<TMPL_IF NAME="SKIN_BG">
  <style>
  html, body {
    background: url("<TMPL_VAR NAME="SKIN_PATH">/common/backgrounds/<TMPL_VAR NAME="SKIN_BG">") no-repeat center fixed;
    background-size: cover;
  }
  </style>
</TMPL_IF>
