
    <TMPL_IF NAME="CHECK_LOGINS">
    <div class="checkbox">
      <label for="checkLogins">
        <input type="checkbox" id="checkLogins" name="checkLogins" <TMPL_IF NAME="ASK_LOGINS">checked</TMPL_IF> />
        <lang en="Check my last logins" fr="Voir mes derni&egrave;res connexions"/>
      </label>
    </div>
    </TMPL_IF>

